@@ -12,12 +12,6 @@ class AbstractSLaMPipeline:
         self.setup_mass = setup_mass
         self.setup_source = setup_source
 
-    @property
-    def shear(self):
-        """For a SLaM source pipeline, determine the shear model from the no_shear setting."""
-        if not self.setup_mass.no_shear:
-            return ag.mp.ExternalShear
-
 
 class SLaMPipelineSourceParametric(AbstractSLaMPipeline):
     def __init__(
@@ -51,28 +45,38 @@ class SLaMPipelineLight(AbstractSLaMPipeline):
 class SLaMPipelineMass(AbstractSLaMPipeline):
     def __init__(
         self,
-        setup_mass: setup.SetupMassTotal = setup.SetupMassTotal(),
-        fix_lens_light=True,
+        setup_mass: ag_setup.AbstractSetupMass = setup.SetupMassTotal(),
+        setup_smbh: ag_setup.SetupSMBH = None,
+        light_is_model=True,
     ):
 
         super().__init__(setup_source=None, setup_light=None, setup_mass=setup_mass)
 
-        self.fix_lens_light = fix_lens_light
+        self.setup_smbh = setup_smbh
+        self.light_is_model = light_is_model
 
     @property
-    def fix_lens_light_tag(self):
+    def light_is_model_tag(self):
         """Generate a tag for if the lens light of the pipeline and / or phase are fixed to a previous estimate,
         or varied during he analysis, to customize phase names.
 
         This changes the setup folder as follows:
 
-        fix_lens_light = False -> setup__
-        fix_lens_light = True -> setup___fix_lens_light
+        light_is_model = False -> setup__
+        light_is_model = True -> setup___light_is_model
         """
-        if not self.fix_lens_light:
+        if not self.light_is_model:
             return ""
-        elif self.fix_lens_light:
-            return f"__{conf.instance.setup_tag.get('pipeline', 'fix_lens_light')}"
+        elif self.light_is_model:
+            return f"__{conf.instance.setup_tag.get('pipeline', 'light_is_model')}"
+
+    @property
+    def smbh_prior_model(self):
+
+        if self.setup_smbh is not None:
+            return self.setup_smbh.smbh_from_centre(
+                centre=af.last.instance.galaxies.lens.sersic.centre
+            )
 
     def shear_from_previous_pipeline(self, index=0):
         """Return the shear _PriorModel_ from a previous pipeline, where:
@@ -164,6 +168,7 @@ class SLaM:
 
         if self.pipeline_light is not None:
             self.pipeline_mass.setup_source = self.pipeline_light.setup_source
+            self.pipeline_mass.setup_light = self.pipeline_light.setup_light
         else:
             if self.pipeline_source_inversion is None:
                 self.pipeline_mass.setup_source = (
@@ -174,6 +179,18 @@ class SLaM:
                     self.pipeline_source_inversion.setup_source
                 )
 
+            if isinstance(self.pipeline_mass.setup_light, ag_setup.SetupLightBulgeDisk):
+
+                self.pipeline_mass.setup_mass.disk_as_sersic = (
+                    self.pipeline_mass.setup_light.disk_as_sersic
+                )
+                self.pipeline_mass.setup_mass.include_envelope = (
+                    self.pipeline_mass.setup_light.include_envelope
+                )
+                self.pipeline_mass.setup_mass.envelope_as_sersic = (
+                    self.pipeline_mass.setup_light.envelope_as_sersic
+                )
+
         self.setup_subhalo = setup_subhalo
 
     @property
@@ -245,6 +262,37 @@ class SLaM:
             return self.source_parametric_tag
         return self.source_inversion_tag
 
+    @property
+    def light_tag(self):
+        """Generate the pipeline's overall tag, which customizes the 'setup' folder the results are output to.
+        """
+
+        setup_tag = conf.instance.setup_tag.get("light", "light")
+        hyper_tag = f"__{self.setup_hyper.tag}" if self.setup_hyper is not None else ""
+
+        if hyper_tag == "__":
+            hyper_tag = ""
+
+        source_tag = (
+            f"__{self.pipeline_light.setup_source.tag}"
+            if self.pipeline_light.setup_source is not None
+            else ""
+        )
+
+        light_tag = (
+            f"__{self.pipeline_light.setup_light.tag}"
+            if self.pipeline_light.setup_light is not None
+            else ""
+        )
+
+        mass_tag = (
+            f"__{self.pipeline_mass.setup_mass.tag}"
+            if self.pipeline_mass.setup_mass is not None
+            else ""
+        )
+
+        return f"{setup_tag}{hyper_tag}{light_tag}{mass_tag}{source_tag}"
+
     @property
     def mass_tag(self):
         """Generate the pipeline's overall tag, which customizes the 'setup' folder the results are output to.
@@ -277,32 +325,184 @@ class SLaM:
 
         return f"{setup_tag}{hyper_tag}{light_tag}{mass_tag}{source_tag}"
 
-    def hyper_galaxy_source_from_previous_pipeline(self, index=0):
+    def link_sersic_light_and_mass_prior_model_from_light_pipeline(
+        self, sersic_prior_model, sersic_is_model=True, index=0
+    ):
 
-        if self.setup_hyper.hyper_galaxies:
+        if sersic_is_model:
 
-            hyper_galaxy = af.PriorModel(ag.HyperGalaxy)
-
-            hyper_galaxy.noise_factor = af.last[
+            sersic_prior_model.centre = af.last[index].model.galaxies.lens.sersic.centre
+            sersic_prior_model.elliptical_comps = af.last[
                 index
-            ].hyper_combined.model.galaxies.source.hyper_galaxy.noise_factor
-            hyper_galaxy.contribution_factor = af.last[
+            ].model.galaxies.lens.sersic.elliptical_comps
+            sersic_prior_model.intensity = af.last[
                 index
-            ].hyper_combined.instance.optional.galaxies.source.hyper_galaxy.contribution_factor
-            hyper_galaxy.noise_power = af.last[
+            ].model.galaxies.lens.sersic.intensity
+            sersic_prior_model.effective_radius = af.last[
                 index
-            ].hyper_combined.instance.optional.galaxies.source.hyper_galaxy.noise_power
+            ].model.galaxies.lens.sersic.effective_radius
+            sersic_prior_model.sersic_index = af.last[
+                index
+            ].model.galaxies.lens.sersic.sersic_index
 
-            return hyper_galaxy
+        else:
 
-    def lens_from_light_pipeline_for_mass_pipeline(self, mass, shear):
+            sersic_prior_model.centre = af.last[
+                index
+            ].instance.galaxies.lens.sersic.centre
+            sersic_prior_model.elliptical_comps = af.last[
+                index
+            ].instance.galaxies.lens.sersic.elliptical_comps
+            sersic_prior_model.intensity = af.last[
+                index
+            ].instance.galaxies.lens.sersic.intensity
+            sersic_prior_model.effective_radius = af.last[
+                index
+            ].instance.galaxies.lens.sersic.effective_radius
+            sersic_prior_model.sersic_index = af.last[
+                index
+            ].instance.galaxies.lens.sersic.sersic_index
+
+    def link_bulge_light_and_mass_prior_model_from_light_pipeline(
+        self, bulge_prior_model, bulge_is_model=True, index=0
+    ):
+
+        if bulge_is_model:
+
+            bulge_prior_model.centre = af.last[index].model.galaxies.lens.bulge.centre
+            bulge_prior_model.elliptical_comps = af.last[
+                index
+            ].model.galaxies.lens.bulge.elliptical_comps
+            bulge_prior_model.intensity = af.last[
+                index
+            ].model.galaxies.lens.bulge.intensity
+            bulge_prior_model.effective_radius = af.last[
+                index
+            ].model.galaxies.lens.bulge.effective_radius
+            bulge_prior_model.sersic_index = af.last[
+                index
+            ].model.galaxies.lens.bulge.sersic_index
+        else:
+
+            bulge_prior_model.centre = af.last[
+                index
+            ].instance.galaxies.lens.bulge.centre
+            bulge_prior_model.elliptical_comps = af.last[
+                index
+            ].instance.galaxies.lens.bulge.elliptical_comps
+            bulge_prior_model.intensity = af.last[
+                index
+            ].instance.galaxies.lens.bulge.intensity
+            bulge_prior_model.effective_radius = af.last[
+                index
+            ].instance.galaxies.lens.bulge.effective_radius
+            bulge_prior_model.sersic_index = af.last[
+                index
+            ].instance.galaxies.lens.bulge.sersic_index
+
+    def link_disk_light_and_mass_prior_model_from_light_pipeline(
+        self, disk_prior_model, disk_is_model=True, index=0
+    ):
+
+        if disk_is_model:
+
+            disk_prior_model.centre = af.last[index].model.galaxies.lens.disk.centre
+            disk_prior_model.elliptical_comps = af.last[
+                index
+            ].model.galaxies.lens.disk.elliptical_comps
+            disk_prior_model.phi = af.last[index].model.galaxies.lens.disk.phi
+            disk_prior_model.intensity = af.last[
+                index
+            ].model.galaxies.lens.disk.intensity
+            disk_prior_model.effective_radius = af.last[
+                index
+            ].model.galaxies.lens.disk.effective_radius
+
+            if self.pipeline_mass.setup_light.disk_as_sersic:
+                disk_prior_model.sersic_index = (
+                    af.last.model.galaxies.lens.disk.sersic_index
+                )
+
+        else:
+
+            disk_prior_model.centre = af.last[index].instance.galaxies.lens.disk.centre
+            disk_prior_model.elliptical_comps = af.last[
+                index
+            ].instance.galaxies.lens.disk.elliptical_comps
+            disk_prior_model.phi = af.last[index].instance.galaxies.lens.disk.phi
+            disk_prior_model.intensity = af.last[
+                index
+            ].instance.galaxies.lens.disk.intensity
+            disk_prior_model.effective_radius = af.last[
+                index
+            ].instance.galaxies.lens.disk.effective_radius
+
+            if self.pipeline_mass.setup_light.disk_as_sersic:
+                disk_prior_model.sersic_index = (
+                    af.last.instance.galaxies.lens.disk.sersic_index
+                )
+
+    def link_envelope_light_and_mass_prior_model_from_light_pipeline(
+        self, envelope_prior_model, envelope_is_model=True, index=0
+    ):
+
+        if not self.pipeline_mass.setup_mass.include_envelope:
+            return
+
+        if envelope_is_model:
+
+            envelope_prior_model.centre = af.last[
+                index
+            ].model.galaxies.lens.envelope.centre
+            envelope_prior_model.elliptical_comps = af.last[
+                index
+            ].model.galaxies.lens.envelope.elliptical_comps
+            envelope_prior_model.phi = af.last[index].model.galaxies.lens.envelope.phi
+            envelope_prior_model.intensity = af.last[
+                index
+            ].model.galaxies.lens.envelope.intensity
+            envelope_prior_model.effective_radius = af.last[
+                index
+            ].model.galaxies.lens.envelope.effective_radius
+
+            if self.pipeline_mass.setup_light.envelope_as_sersic:
+                envelope_prior_model.sersic_index = (
+                    af.last.model.galaxies.lens.envelope.sersic_index
+                )
+
+        else:
+
+            envelope_prior_model.centre = af.last[
+                index
+            ].instance.galaxies.lens.envelope.centre
+            envelope_prior_model.elliptical_comps = af.last[
+                index
+            ].instance.galaxies.lens.envelope.elliptical_comps
+            envelope_prior_model.phi = af.last[
+                index
+            ].instance.galaxies.lens.envelope.phi
+            envelope_prior_model.intensity = af.last[
+                index
+            ].instance.galaxies.lens.envelope.intensity
+            envelope_prior_model.effective_radius = af.last[
+                index
+            ].instance.galaxies.lens.envelope.effective_radius
+
+            if self.pipeline_mass.setup_light.envelope_as_sersic:
+                envelope_prior_model.sersic_index = (
+                    af.last.instance.galaxies.lens.envelope.sersic_index
+                )
+
+    def _lens_from_light_sersic_pipeline_for_mass_pipeline(
+        self, mass, shear, light_is_model
+    ):
         """Setup the lens model for a Mass pipeline using the previous pipeline and phase results.
 
         The lens light model is not specified by the Mass pipeline, so the Light pipelines are used to
         determine this. This function returns a _GalaxyModel_ for the lens, where:
 
         1) The lens light model uses the light model of the Light pipeline.
-        2) The lens light is returned as a model if *fix_lens_light* is *False, an instance if *True*.
+        2) The lens light is returned as a model if *light_is_model* is *False, an instance if *True*.
 
         Parameters
         ----------
@@ -314,12 +514,54 @@ class SLaM:
             The _ExternalShear_ of the lens galaxy.
         """
 
-        if self.pipeline_mass.fix_lens_light:
+        if not light_is_model:
+
+            return ag.GalaxyModel(
+                redshift=self.redshift_lens,
+                sersic=af.last.instance.galaxies.lens.sersic,
+                mass=mass,
+                shear=shear,
+                hyper_galaxy=af.last.hyper_combined.instance.optional.galaxies.lens.hyper_galaxy,
+            )
+
+        else:
+
+            return ag.GalaxyModel(
+                redshift=self.redshift_lens,
+                serssic=af.last.model.galaxies.lens.sersic,
+                mass=mass,
+                shear=shear,
+                hyper_galaxy=af.last.hyper_combined.instance.optional.galaxies.lens.hyper_galaxy,
+            )
+
+    def _lens_from_light_bulge_disk_pipeline_for_mass_pipeline(
+        self, mass, shear, light_is_model
+    ):
+        """Setup the lens model for a Mass pipeline using the previous pipeline and phase results.
+
+        The lens light model is not specified by the Mass pipeline, so the Light pipelines are used to
+        determine this. This function returns a _GalaxyModel_ for the lens, where:
+
+        1) The lens light model uses the light model of the Light pipeline.
+        2) The lens light is returned as a model if *light_is_model* is *False, an instance if *True*.
+
+        Parameters
+        ----------
+        redshift_lens : float
+            The redshift of the lens galaxy.
+        mass : ag.MassProfile
+            The mass model of the len galaxy.
+        shear : ag.ExternalShear
+            The _ExternalShear_ of the lens galaxy.
+        """
+
+        if not light_is_model:
 
             return ag.GalaxyModel(
                 redshift=self.redshift_lens,
                 bulge=af.last.instance.galaxies.lens.bulge,
                 disk=af.last.instance.galaxies.lens.disk,
+                envelope=af.last.instance.galaxies.lens.envelope,
                 mass=mass,
                 shear=shear,
                 hyper_galaxy=af.last.hyper_combined.instance.optional.galaxies.lens.hyper_galaxy,
@@ -331,11 +573,43 @@ class SLaM:
                 redshift=self.redshift_lens,
                 bulge=af.last.model.galaxies.lens.bulge,
                 disk=af.last.model.galaxies.lens.disk,
+                envelope=af.last.model.galaxies.lens.envelope,
                 mass=mass,
                 shear=shear,
                 hyper_galaxy=af.last.hyper_combined.instance.optional.galaxies.lens.hyper_galaxy,
             )
 
+    def lens_from_light_pipeline_for_mass_pipeline(self, mass, shear):
+        """Setup the lens model for a Mass pipeline using the previous pipeline and phase results.
+
+        The lens light model is not specified by the Mass pipeline, so the Light pipelines are used to
+        determine this. This function returns a _GalaxyModel_ for the lens, where:
+
+        1) The lens light model uses the light model of the Light pipeline.
+        2) The lens light is returned as a model if *light_is_model* is *False, an instance if *True*.
+
+        Parameters
+        ----------
+        redshift_lens : float
+            The redshift of the lens galaxy.
+        mass : ag.MassProfile
+            The mass model of the len galaxy.
+        shear : ag.ExternalShear
+            The _ExternalShear_ of the lens galaxy.
+        """
+
+        if isinstance(self.pipeline_mass.setup_light, setup.SetupLightBulgeDisk):
+
+            return self._lens_from_light_bulge_disk_pipeline_for_mass_pipeline(
+                mass=mass, shear=shear, light_is_model=self.pipeline_mass.light_is_model
+            )
+
+        else:
+
+            return self._lens_from_light_sersic_pipeline_for_mass_pipeline(
+                mass=mass, shear=shear, light_is_model=self.pipeline_mass.light_is_model
+            )
+
     def lens_for_subhalo_pipeline(self, index=0):
 
         if self.setup_subhalo.mass_is_model:
@@ -356,9 +630,11 @@ class SLaM:
                 hyper_galaxy=af.last.hyper_combined.instance.optional.galaxies.lens.hyper_galaxy,
             )
 
-    def source_sersic_from_previous_pipeline(self, source_is_model=True, index=0):
+    def _source_sersic_from_previous_pipeline(self, source_is_model=True, index=0):
 
-        hyper_galaxy = self.hyper_galaxy_source_from_previous_pipeline(index=index)
+        hyper_galaxy = self.setup_hyper.hyper_galaxy_source_from_previous_pipeline(
+            index=index
+        )
 
         if source_is_model:
 
@@ -376,9 +652,11 @@ class SLaM:
                 hyper_galaxy=hyper_galaxy,
             )
 
-    def source_inversion_from_previous_pipeline(self, source_is_model=False, index=0):
+    def _source_inversion_from_previous_pipeline(self, source_is_model=False, index=0):
 
-        hyper_galaxy = self.hyper_galaxy_source_from_previous_pipeline(index=index)
+        hyper_galaxy = self.setup_hyper.hyper_galaxy_source_from_previous_pipeline(
+            index=index
+        )
 
         if source_is_model:
 
@@ -431,13 +709,13 @@ class SLaM:
 
         if self.pipeline_source_inversion is None:
 
-            return self.source_sersic_from_previous_pipeline(
+            return self._source_sersic_from_previous_pipeline(
                 source_is_model=source_is_model, index=index
             )
 
         else:
 
-            return self.source_inversion_from_previous_pipeline(
+            return self._source_inversion_from_previous_pipeline(
                 source_is_model=source_is_model, index=index
             )
 
