@@ -25,8 +25,8 @@ workspace_path = os.environ["WORKSPACE"]
 print("Workspace Path: ", workspace_path)
 
 conf.instance = conf.Config(
-    config_path=f"{workspace_path}/howtolens/config",
-    output_path=f"{workspace_path}/howtolens/output",
+    config_path=f"{workspace_path}/config",
+    output_path=f"{workspace_path}/output",
 )
 
 # %%
@@ -45,11 +45,11 @@ We'll use new strong lensing data, where:
 """
 
 # %%
-from autolens_workspace.howtolens.simulators.chapter_3 import mass_sie__source_sersic_x4
+from howtolens.simulators.chapter_3 import mass_sie__source_sersic_x4
 
 dataset_type = "chapter_3"
 dataset_name = "mass_sie__source_sersic_x4"
-dataset_path = f"{workspace_path}/howtolens/dataset/{dataset_type}/{dataset_name}"
+dataset_path = f"{workspace_path}/dataset/{dataset_type}/{dataset_name}"
 
 imaging = al.Imaging.from_fits(
     image_path=f"{dataset_path}/image.fits",
@@ -133,7 +133,7 @@ The 'folders' below specify the path the pipeline results are written to, which
 """
 
 # %%
-from autolens_workspace.howtolens.chapter_3_pipelines import (
+from howtolens.chapter_3_pipelines import (
     tutorial_3_pipeline_complex_source,
 )
 
